import { Component } from '@angular/core';
import { AppPage, AppPageState } from '@app/shared/pages/base-page.class';
import { RxState } from '@rx-angular/state';
import { Promise } from '@rx-angular/cdk/zone-less/browser';
import { TranslateService } from '@ngx-translate/core';
import { ModalController } from '@ionic/angular';

interface CertificationVerificationStep {
  labelKey: string;
  expectedResponse: boolean;
}
interface ModalSequenceConfig {
  title: string;
  steps: CertificationVerificationStep[];
}

export interface VerificationModalState extends AppPageState {}

@Component({
  selector: 'app-verification-modal',
  templateUrl: './verification.modal.html',
  styleUrls: ['./verification.modal.css'],
  providers: [RxState],
})
export class VerificationModal extends AppPage<VerificationModalState> {
  private readonly certificationSteps: CertificationVerificationStep[] = [
    {
      labelKey: 'ACCOUNT.CERTIFICATION_MODAL.QUESTIONS.WELL_KNOWN',
      expectedResponse: true,
    },
    {
      labelKey: 'ACCOUNT.CERTIFICATION_MODAL.QUESTIONS.REVOCATION',
      expectedResponse: true,
    },
    {
      labelKey: 'ACCOUNT.CERTIFICATION_MODAL.QUESTIONS.CONTACT',
      expectedResponse: true,
    },
    {
      labelKey: 'ACCOUNT.CERTIFICATION_MODAL.QUESTIONS.DOUBLE_IDENTITY',
      expectedResponse: false,
    },
    {
      labelKey: 'ACCOUNT.CERTIFICATION_MODAL.QUESTIONS.MASTER_ACCOUNT',
      expectedResponse: true,
    },
    {
      labelKey: 'ACCOUNT.CERTIFICATION_MODAL.QUESTIONS.LICENSE',
      expectedResponse: true,
    },
    {
      labelKey: 'ACCOUNT.CERTIFICATION_MODAL.QUESTIONS.CREDENTIALS',
      expectedResponse: true,
    },
    {
      labelKey: 'ACCOUNT.CERTIFICATION_MODAL.QUESTIONS.PUBLIC_KEY_DIFFERENT',
      expectedResponse: false,
    },
  ];

  answers: Map<string, boolean> = new Map<string, boolean>();
  certificationModalConfig: ModalSequenceConfig;

  constructor(
    protected translate: TranslateService,
    protected viewCtrl: ModalController
  ) {
    super({ name: 'verification-modal' });
    this.certificationModalConfig = this.getCertificationModalConfig();
  }

  protected ngOnLoad(): Promise<Partial<VerificationModalState>> {
    return Promise.resolve(undefined);
  }

  private getCertificationModalConfig(): ModalSequenceConfig {
    return {
      title: 'ACCOUNT.CERTIFICATION_MODAL.CHECKLIST_TITLE',
      steps: this.shuffleSteps(this.certificationSteps),
    };
  }

  private shuffleSteps<T>(array: T[]): T[] {
    const shuffled = [...array];
    for (let i = shuffled.length - 1; i > 0; i--) {
      const j = Math.floor(Math.random() * (i + 1));
      [shuffled[i], shuffled[j]] = [shuffled[j], shuffled[i]];
    }
    return shuffled.slice(0, 4);
  }

  onAnswerSelect(step: string, value: boolean) {
    this.answers.set(step, value);
  }

  cancel() {
    return this.viewCtrl.dismiss(undefined);
  }

  isCompleted(): boolean {
    return this.certificationModalConfig.steps.length === this.answers.size;
  }

  doSubmit() {
    return this.viewCtrl.dismiss(
      this.isCompleted() && this.certificationModalConfig.steps.every((step) => this.answers.get(step.labelKey) === step.expectedResponse)
    );
  }
}
