import { Component, Input } from '@angular/core';
import { AppPage, AppPageState } from '@app/shared/pages/base-page.class';
import { RxState } from '@rx-angular/state';
import { Promise } from '@rx-angular/cdk/zone-less/browser';
import { TranslateService } from '@ngx-translate/core';
import { environment } from '@environments/environment';
import { Router } from '@angular/router';
import { NetworkService } from '@app/network/network.service';
import { JsonTopic } from '@app/home/feed/feed.model';
import { SettingsService } from '@app/settings/settings.service';
import { isNotNil } from '@app/shared/functions';

export interface FeedState extends AppPageState {
  topics: JsonTopic[];
}

@Component({
  selector: 'app-feeds',
  templateUrl: './feed.component.html',
  styleUrls: ['./feed.component.scss'],
  providers: [RxState],
})
export class FeedComponent extends AppPage<FeedState> {
  @Input() showToolbar = true;

  topics: JsonTopic[];
  version: string;

  constructor(
    protected translate: TranslateService,
    protected settings: SettingsService,
    protected networkService: NetworkService,
    protected router: Router
  ) {
    super({ name: 'about-modal' });
  }

  protected async ngOnLoad(): Promise<FeedState> {
    await Promise.all([this.settings.ready(), this.networkService.ready()]);
    const feedUrls = environment.feed?.jsonFeed[this.settings.locale];
    this.topics = await this.networkService.loadJsonFeedUrls(feedUrls);
    this.topics = this.topics.filter(isNotNil).map((value) => value);
    return <FeedState>{ topics: this.topics };
  }

  redirectToForum(url: string) {
    window.open(url.replace(/\.json$/, ''), '_blank');
  }

  redirectToForumHome() {
    window.open(environment.forumUrl, '_blank');
  }

  protected readonly environment = environment;
}
