import { Component, Input, OnInit } from '@angular/core';
import { AppForm } from '@app/shared/form.class';
import { FormBuilder, Validators } from '@angular/forms';
import { environment } from '@environments/environment';
import { setTimeout } from '@rx-angular/cdk/zone-less/browser';

export interface ValidationData {
  codeValidation?: string;
}
@Component({
  selector: 'app-transfer-validation-form',
  templateUrl: './validation.form.html',
  styleUrls: ['./validation.form.scss'],
})
export class ValidationForm extends AppForm<ValidationData> implements OnInit {
  @Input() message: string;

  showPwd = false;

  constructor(formBuilder: FormBuilder) {
    super(
      formBuilder.group({
        codeValidation: [null, Validators.required],
      })
    );
    this._enable = true;
    this.debug = !environment.production;
  }

  ngOnInit() {
    super.ngOnInit();
  }

  protected toggleShowPwd(event?: Event) {
    event?.preventDefault();
    this.showPwd = !this.showPwd;
    this.markForCheck();

    // Auto hide
    if (this.showPwd) {
      setTimeout(() => {
        if (this.showPwd) {
          this.showPwd = false;
          this.markForCheck();
        }
      }, 2000);
    }
  }

  get value(): ValidationData {
    const json = this.form.value;
    return {
      codeValidation: json.codeValidation,
    };
  }

  get invalid() {
    return this.form?.invalid;
  }
  get valid() {
    return this.form?.valid;
  }
}
