// This file can be replaced during build by using the `fileReplacements` array.
// `ng build --prod` replaces `environment.ts` with `environment.prod.ts`.
// The list of file replacements can be found in `angular.json`.

import { Environment } from './environment.class';
import { StorageDrivers } from '@app/shared/services/storage/storage.utils';
import { AuthData } from '@app/account/auth/auth.model';

export const environment = <Environment>{
  production: false,
  version: '2.0.1',
  name: 'Cesium&sup2;',
  mode: 'dev',
  defaultLocale: 'fr',
  forumInformationUrl: 'https://forum.duniter.org/t/gchange-v2-moderation-ou-et-federation/12910/2',

  // About
  officialUrl: 'https://www.cesium.app',
  sourceUrl: 'https://git.duniter.org/clients/cesium-grp/cesium',
  reportIssueUrl: 'https://git.duniter.org/clients/cesium-grp/cesium/-/issues/new',
  forumUrl: 'https://forum.monnaie-libre.fr',

  graphql: {
    fetchPolicy: 'cache-first',
    watchFetchPolicy: 'cache-and-network',
    persistCache: false,
    fetchSize: 20,
  },

  // Storage
  storage: {
    name: 'cesium2',
    driverOrder: [StorageDrivers.IndexedDB, StorageDrivers.WebSQL, StorageDrivers.LocalStorage],
  },

  keyring: {
    ss58Format: 42, // dev
  },

  feed: {
    jsonFeed: {
      fr: ['https://forum.monnaie-libre.fr/t/actu-generale-la-1-bouge/31318.json'],
    },
    maxContentLength: 1300,
    maxAgeInMonths: 3,
    maxCount: 1,
  },

  dev: {
    //peer: 'ws://127.0.0.1:9944',
    peer: 'wss://gdev.coinduf.eu/ws',
    //peer: 'wss://gdev.cgeek.fr/ws',
    indexer: 'https://squid.gdev.coinduf.eu/v1beta1/relay',

    auth: <AuthData>{
      v1: {
        salt: 'test',
        password: 'test',
      },
      meta: {
        name: 'Compte test v1',
        default: false,
      },
    },
  },

  defaultPeers: [
    /* Local endpoint */
    //'ws://127.0.0.1:9944',
    /* GDev endpoints */
    // 'wss://vit.fdn.org/ws',
    //'wss://1000i100.fr/ws',
    'wss://gdev.coinduf.eu/ws',
    'wss://gdev.pini.fr/ws',
    'wss://gdev.cgeek.fr/ws',
    'wss://gdev.p2p.legal/ws',
    'wss://archive-rpc.gdev.brussels.ovh/ws',
    'wss://archive-rpc.gdev.de.brussels.ovh/ws',
    'wss://gdev.p2p.legal/ws',
  ],

  defaultIndexers: [
    /* Local endpoint */
    //'http://localhost:8080/v1/graphql',
    /* GDev endpoints */
    'https://squid.gdev.coinduf.eu/v1beta1/relay',
    // 'https://gdev-squid.axiom-team.fr/graphql',
    'https://gdev-squid.axiom-team.fr/v1beta1/relay',
  ],

  defaultPods: [
    /* Local endpoint */
    // 'http://localhost:8081/v1/graphql'
    /* GDev endpoints */
    'https://datapod.coinduf.eu/v1/graphql',
    'https://datapod.gyroi.de:443/v1/graphql',
  ],

  defaultIfpsGateways: [
    /* Local endpoint */
    // 'http://localhost:8080'
    /* GDev endpoints */
    'https://pagu.re',
    'https://gyroi.de',
  ],

  defaultDistanceFileUrls: [
    /* Distance file Urls */
    'https://files.coinduf.eu/distance_precompute/latest_distance.json',
  ],
};
